#include <oslib/oslib.h>
#include "sprites.h"

PSP_MODULE_INFO("OSLib Sprites", 0, 1, 1);
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER | THREAD_ATTR_VFPU);

#define SCREEN_WIDTH 480
#define SCREEN_HEIGHT 272

OSL_IMAGE *mummy, *mnky, *link, *alpha, *fx, *drop, *worm;
SPRITE *mummyspt, *mnkyspt, *linkspt, *alphaspt, *expspt, *dropspt, *wormspt;

int i, linklastdir, alphastart = 1, alphaend = 10, dropx = 100, expx = 100, expy = 100;

void Keys();
void Text();
void Draw();
void initSprites();

int main(int argc, char* argv[])
{
	//Initialize OSLib
	oslInit(0);
	oslInitGfx(OSL_PF_8888, 1);
	oslInitConsole();

	//Show Contest Logo
	oslShowNeoflashLogo();

	//Configure the joypad
	oslSetKeyAutorepeatInit(40);
	oslSetKeyAutorepeatInterval(10);

	//Setup the sprites
	initSprites();

	while (!osl_quit)
	{
		//start drawing
		oslStartDrawing();

		//read keys
		Keys();

		//draw sprites
		Draw();

		//end drawing and flip buffer
		oslEndDrawing();
		oslSyncFrame();
	}

	//clean up
	oslEndGfx();
	oslQuit();
	return 0;
}

void initSprites(){

	//load images
	mummy = oslLoadImageFile("mummy.png", OSL_IN_RAM, OSL_PF_5551);
	mnky = oslLoadImageFile("monkey.png", OSL_IN_RAM, OSL_PF_5551);
	link = oslLoadImageFile("link.png", OSL_IN_RAM, OSL_PF_5551);
	alpha = oslLoadImageFile("alpha.png", OSL_IN_RAM, OSL_PF_5551);
	fx = oslLoadImageFile("sonicfx.png", OSL_IN_RAM, OSL_PF_5551);
	if (!mummy || !mnky || !link || !alpha || !fx)
		oslFatalError("At least one image is missing. Please copy all the images to the GAME directory.");

	//monkey
	mnkyspt = createSprite(mnky, 56, 34, 3);
	mnky->x = SCREEN_WIDTH - 70;
	mnky->y = 15;

	//alpha
	alphaspt = createSprite(alpha, 8, 16, 16);
	alpha->x = SCREEN_WIDTH - 40;
	alpha->y = SCREEN_HEIGHT - 40;

	//mummy
	mummyspt = createSprite(mummy, 37, 45, 5);
	spriteSetFrameBounds(mummyspt, 1, 18);
	mummy->centerX = mummyspt->width / 2;
	mummy->centerY = mummyspt->height / 2;
	mummy->x = 240;
	mummy->y = 135;

	//explosion
	expspt = createSprite(fx, 32, 32, 5);
	spriteSetFrameBounds(expspt, 1, 12);

	//drop
	drop = oslCreateImageTile(fx,30,82,150,182);
	dropspt = createSprite(drop, 30, 50, 7);

	//worm
	worm = oslCreateImageTile(fx,150,82,342,130);
	wormspt = createSprite(worm, 24, 24, 7);

	//link
	linkspt = createSprite(link, 25, 25, 5);
	link->x = linkspt->width / 2;
	link->y = SCREEN_HEIGHT - 100;
	link->centerX = linkspt->width / 2;
	link->centerY = linkspt->height / 2;
	//define some constants for link
	#define LINK_START 2
	#define LINK_END 6
	#define LINK_LR 1
	#define LINK_DOWN 2
	#define LINK_UP 3
	#define LINK_DIST 2
	linklastdir = LINK_DOWN;

}
void Keys()
{
	oslReadKeys();

	//alpha
	if( osl_keys->pressed.up && osl_keys->held.L && alphastart < alphaspt->max){
		alphastart++;
	} else if( osl_keys->pressed.up && osl_keys->held.R && alphaend < alphaspt->max){
		alphaend++;
	}
	if( osl_keys->pressed.down && osl_keys->held.L && alphastart > 1){
		alphastart--;
	} else if( osl_keys->pressed.down && osl_keys->held.R && alphaend > 1){
		alphaend--;
	}
	if (osl_keys->pressed.triangle){ alphaspt->reverse = alphaspt->reverse ? 0 : 1; }

	//mummy walks or runs?
	int factor = osl_keys->held.circle ? 2 : 1;
	if( factor == 2 ){ mummyspt->interval=3; }
	else{ mummyspt->interval=5; }

	//play/stop by default
	spritePlay(mummyspt);
	spritePlay(linkspt);
	spriteStop(mnkyspt);

	//pad up
	if (osl_keys->held.up){

		//mummy
		spriteSetFrame(mummyspt, 20);
		spriteStop(mummyspt);

		//monkey
		if(mummy->x > 375){
			spritePlay(mnkyspt);
		}

		//link
		linkspt->fliph=0;
		link->y -= LINK_DIST;
		if(link->y < -linkspt->height){ link->y = SCREEN_HEIGHT; }
		spriteSetFrameBounds(linkspt, (LINK_UP-1)*linkspt->cols+LINK_START, (LINK_UP-1)*linkspt->cols+LINK_END);
		linklastdir = LINK_UP;

	//pad down
	} else if (osl_keys->held.down){

		//mummy
		spriteSetFrame(mummyspt, 19);
		spriteStop(mummyspt);

		//link
		linkspt->fliph=0;
		link->y += LINK_DIST;
		if(link->y > SCREEN_HEIGHT){ link->y = 0; }
		spriteSetFrameBounds(linkspt, (LINK_DOWN-1)*linkspt->cols+LINK_START, (LINK_DOWN-1)*linkspt->cols+LINK_END);
		linklastdir = LINK_DOWN;

	} else {

		//pad left/right
		if( osl_keys->held.left || osl_keys->held.right ){

			//move mummy/link, wrap screen
			if(osl_keys->held.left){

				//mummy
				mummy->x -= factor;
				if(mummy->x < -mummyspt->width){ mummy->x = SCREEN_WIDTH; }
				mummyspt->fliph=1;

				//link
				linkspt->fliph=1;
				link->x -= LINK_DIST;
				if(link->x < -linkspt->width){ link->x = SCREEN_WIDTH; }

			} else if(osl_keys->held.right){

				//mummy
				mummy->x += factor;
				if(mummy->x > SCREEN_WIDTH){ mummy->x = -1*mummyspt->width; }
				mummyspt->fliph=0;

				//link
				linkspt->fliph=0;
				link->x += LINK_DIST;
				if(link->x > SCREEN_WIDTH){ link->x = -1*linkspt->width; }

			}

			//set the correct frames for link
			spriteSetFrameBounds(linkspt, LINK_LR*LINK_START, LINK_LR*LINK_END);
			//record link direction
			linklastdir = LINK_LR;

		//otherwise set to a "still" frame
		} else {
			spriteSetFrame(mummyspt, 5);
			spriteStop(mummyspt);
			spriteSetFrame(linkspt, linkspt->cols*(linklastdir-1)+1);
			spriteStop(linkspt);
		}

	}

	if (osl_keys->pressed.start) oslQuit(); //exit the sample
}

void Draw()
{
	//clear the screen
	oslClearScreen(RGBA(0,0,0,255));

	//draw the text			
	Text();

	//draw the sprites
	spriteDraw(mummyspt);
	spriteDraw(mnkyspt);
	spriteDraw(linkspt);

	spriteSetFrameBounds(alphaspt, alphastart, alphaend);
	spriteDraw(alphaspt);

	spriteDrawXY(dropspt,dropx,SCREEN_HEIGHT - 65);
	if( dropspt->current == dropspt->end ){
		dropx = (dropx < 400) ? (dropx + 30) : 100;
	}

	spriteDrawXY(expspt,expx,expy);
	if( expspt->current == expspt->end ){
		expx = (rand() % 400) + 40;
		expy = (rand() % 230) + 40;
	}

	spriteDrawXY(wormspt,15,SCREEN_HEIGHT - 40);

}

void Text()
{
	//set the color of the text
	oslSetTextColor(RGBA(255,255,255,255));
	oslSetBkColor(RGBA(0,0,0,0));
	//print instructions
	oslPrintf_xy(10,10,"-= OSLib Sprites Example =-");
	oslPrintf_xy(10,25," + Joypad to move mummy and link");
	oslPrintf_xy(10,35,"(O) to make mummy run");
	oslPrintf_xy(10,45,"(^) to reverse alpha animation");
	oslPrintf_xy(10,55,"[ L ] plus up or down to adjust alpha start frame");
	oslPrintf_xy(10,65,"[ R ] plus up or down to adjust alpha end frame");
	oslPrintf_xy(10,75,"walk to monkey and look up to make him dance");
	oslPrintf_xy(10,95,"[Start] to Quit");
}
